/**  \file  suite_oomfill.c
 *  \brief  Unit testing for oomfill helper
 * \author  François Cerbelle (Fanfan), francois@cerbelle.net
 *
 * \internal
 *       Created:  07/07/2024
 *      Revision:  none
 * Last modified:  2024-12-10 22:02
 *      Compiler:  gcc
 *  Organization:  Cerbelle.net
 *     Copyright:  Copyright (c) 2024, François Cerbelle
 *
 *  This source code is released for free distribution under the terms of the
 *  GNU General Public License as published by the Free Software Foundation.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#define _XOPEN_SOURCE 500                       /* snprintf */
#include "liboom/oomfill.h"                     /* OOM simulation */
#include <check.h>
#include <unistd.h>                             /* pid_t getpid() */
#include <sys/stat.h>                           /* mkdir(), chdir(), mode_t */
#include <stdio.h>                              /* fprintf */
#include <stdlib.h>                             /* abort() */
#include <sys/resource.h>                       /* setrlimit */
#include <errno.h>                              /* errno */
#include <string.h>                             /* strerror() */
#include <limits.h>                             /* ULONG_MAX */
#include "checktools.inc"

START_TEST(oomfill_config_once_0) {
    struct rlimit limit;
    size_t l_result;
    size_t l_avail;

    /* Find installed physical RAM, 0 in case of failure */
    l_avail = (sysconf(_SC_PHYS_PAGES) * sysconf(_SC_PAGESIZE));

    /* If pages were freed by the OS since a previous setrlimit, we need to use
     * the smallest value between previous rlimit value and available pages */
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    l_avail=(limit.rlim_max<l_avail?limit.rlim_max:l_avail);

    l_result=oomfill_config(0);
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    ck_assert(limit.rlim_cur==limit.rlim_max);
    ck_assert(l_result==limit.rlim_max);
    ck_assert(l_result==l_avail);
}
END_TEST

START_TEST(oomfill_config_once_max) {
    struct rlimit limit;
    size_t l_result;
    size_t l_avail;

    /* Find installed physical RAM, 0 in case of failure */
    l_avail = (sysconf(_SC_PHYS_PAGES) * sysconf(_SC_PAGESIZE));

    /* If pages were freed by the OS since a previous setrlimit, we need to use
     * the smallest value between previous rlimit value and available pages */
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    l_avail=(limit.rlim_max<l_avail?limit.rlim_max:l_avail);

    l_result=oomfill_config(l_avail);
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    ck_assert(limit.rlim_cur==limit.rlim_max);
    ck_assert(l_result==limit.rlim_max);
    ck_assert(l_result==l_avail);
}
END_TEST

START_TEST(oomfill_config_once_over) {
    size_t l_avail;

    /* This test is supposed to trigger a SIGABRT(6) and will crash the
     * whole testsuite if not caught or not in a child process */
    forktest_only;

    /* Find installed physical RAM, 0 in case of failure */
    l_avail = (sysconf(_SC_PHYS_PAGES) * sysconf(_SC_PAGESIZE));

    /* SIGABRT(6) expected here */
    oomfill_config(l_avail+1);
}
END_TEST

START_TEST(oomfill_config_once_value) {
    struct rlimit limit;
    size_t l_result;
    size_t l_avail;

    /* Find available physical RAM, 0 in case of failure */
    l_avail = (sysconf(_SC_AVPHYS_PAGES) * sysconf(_SC_PAGESIZE));

    /* If pages were freed by the OS since a previous setrlimit, we need to use
     * the smallest value between previous rlimit value and available pages */
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    l_avail=(limit.rlim_max<l_avail?limit.rlim_max:l_avail);

    l_result=oomfill_config(l_avail/2);
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    ck_assert(limit.rlim_cur==limit.rlim_max);
    ck_assert(l_result==limit.rlim_max);
    ck_assert(l_result==l_avail/2);
}
END_TEST

START_TEST(oomfill_config_twice_same) {
    struct rlimit limit;
    size_t l_result;
    size_t l_avail;

    /* Find available physical RAM, 0 in case of failure */
    l_avail = (sysconf(_SC_AVPHYS_PAGES) * sysconf(_SC_PAGESIZE));

    /* If pages were freed by the OS since a previous setrlimit, we need to use
     * the smallest value between previous rlimit value and available pages */
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    l_avail=(limit.rlim_max<l_avail?limit.rlim_max:l_avail);

    /* First config, already tested */
    l_result=oomfill_config(l_avail/2);
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    ck_assert(limit.rlim_cur==limit.rlim_max);
    ck_assert(l_result==limit.rlim_max);
    ck_assert(l_result==l_avail/2);

    /* Second config, actuel current test */
    l_result=oomfill_config(l_avail/2);
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    ck_assert(limit.rlim_cur==limit.rlim_max);
    ck_assert(l_result==limit.rlim_max);
    ck_assert(l_result==l_avail/2);
}
END_TEST

START_TEST(oomfill_config_twice_up) {
    struct rlimit limit;
    size_t l_result;
    size_t l_avail;

    /* This test is supposed to trigger a SIGABRT(6) and will crash the
     * whole testsuite if not caught or not in a child process */
    forktest_only;

    /* Find available physical RAM, 0 in case of failure */
    l_avail = (sysconf(_SC_AVPHYS_PAGES) * sysconf(_SC_PAGESIZE));

    /* If pages were freed by the OS since a previous setrlimit, we need to use
     * the smallest value between previous rlimit value and available pages */
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    l_avail=(limit.rlim_max<l_avail?limit.rlim_max:l_avail);

    /* First configuration to a third of the available RAM, already tested */
    l_result=oomfill_config(l_avail/3);
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    ck_assert(l_result==limit.rlim_max);
    ck_assert(l_result==l_avail/3);

    /* Second config to set hard limit to half of the available RAM (higher) */
    /* SIGABRT(6) expected here */
    oomfill_config(l_avail/2);
}
END_TEST

START_TEST(oomfill_config_twice_down) {
    struct rlimit limit;
    size_t l_result;
    size_t l_avail;

    /* Find available physical RAM, 0 in case of failure */
    l_avail = (sysconf(_SC_AVPHYS_PAGES) * sysconf(_SC_PAGESIZE));

    /* If pages were freed by the OS since a previous setrlimit, we need to use
     * the smallest value between previous rlimit value and available pages */
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    l_avail=(limit.rlim_max<l_avail?limit.rlim_max:l_avail);

    l_result=oomfill_config(l_avail/2);
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    ck_assert(limit.rlim_cur==limit.rlim_max);
    ck_assert(l_result==limit.rlim_max);
    ck_assert(l_result==l_avail/2);

    l_result=oomfill_config(l_avail/3);
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    ck_assert(limit.rlim_cur==limit.rlim_max);
    ck_assert(l_result==limit.rlim_max);
    ck_assert(l_result==l_avail/3);
}
END_TEST

START_TEST(oomfill_noconfig_enable_0) {
    /* This test is supposed to trigger a SIGABRT(6) and will crash the
     * whole testsuite if not caught or not in a child process */
    /* Furthermore, a previous test already configured the limit for the
     * process if not forked */
    forktest_only;

    ck_assert(0==oomfill_enabled());
    /* SIG6 expected */
    oomfill_enable(0);
}
END_TEST

START_TEST(oomfill_noconfig_enable_value) {
    struct rlimit limit;
    size_t l_avail;

    /* This test is supposed to trigger a SIGABRT(6) and will crash the
     * whole testsuite if not caught or not in a child process */
    /* Furthermore, a previous test already configured the limit for the
     * process if not forked */
    forktest_only;

    /* Find available physical RAM, 0 in case of failure */
    l_avail = (sysconf(_SC_AVPHYS_PAGES) * sysconf(_SC_PAGESIZE));

    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    ck_assert(limit.rlim_cur==limit.rlim_max);
    l_avail=(limit.rlim_max<l_avail?limit.rlim_max:l_avail);

    ck_assert(0==oomfill_enabled());
    /* SIG6 expected */
    oomfill_enable(l_avail/2);
}
END_TEST

START_TEST(oomfill_noconfig_enable_max) {
    struct rlimit limit;
    size_t l_avail;

    /* This test is supposed to trigger a SIGABRT(6) and will crash the
     * whole testsuite if not caught or not in a child process */
    /* Furthermore, a previous test already configured the limit for the
     * process if not forked */
    forktest_only;

    /* Find installed physical RAM, 0 in case of failure */
    l_avail = (sysconf(_SC_PHYS_PAGES) * sysconf(_SC_PAGESIZE));

    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    ck_assert(limit.rlim_cur==limit.rlim_max);
    l_avail=(limit.rlim_max<l_avail?limit.rlim_max:l_avail);

    ck_assert(0==oomfill_enabled());
    /* SIG6 expected */
    oomfill_enable(l_avail);
}
END_TEST

START_TEST(oomfill_noconfig_enable_over) {
    struct rlimit limit;
    size_t l_avail;

    /* This test is supposed to trigger a SIGABRT(6) and will crash the
     * whole testsuite if not caught or not in a child process */
    /* Furthermore, a previous test already configured the limit for the
     * process if not forked */
    forktest_only;

    /* Find installed physical RAM, 0 in case of failure */
    l_avail = (sysconf(_SC_PHYS_PAGES) * sysconf(_SC_PAGESIZE));

    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    ck_assert(limit.rlim_cur==limit.rlim_max);
    l_avail=(limit.rlim_max<l_avail?limit.rlim_max:l_avail);

    ck_assert(0==oomfill_enabled());
    /* SIG6 expected */
    oomfill_enable(l_avail+1);
}
END_TEST

START_TEST(oomfill_config_enable_0) {
    struct rlimit limit;
    size_t l_result;
    size_t l_avail;

    /* Find available physical RAM, 0 in case of failure */
    l_avail = (sysconf(_SC_AVPHYS_PAGES) * sysconf(_SC_PAGESIZE));

    /* Configure a hard limit only if not already configured by non-forked
     * previous tests */
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    ck_assert(limit.rlim_cur==limit.rlim_max);
    l_avail=(limit.rlim_max<l_avail?limit.rlim_max:l_avail);
    oomfill_config(l_avail);

    /* Soft limit to hard limit */
    ck_assert(0==oomfill_enabled());
    l_result=oomfill_enable(0);

    /* Check */
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    ck_assert(limit.rlim_cur==l_avail);
    ck_assert(limit.rlim_max==l_avail);
    ck_assert(l_result==limit.rlim_cur);
    ck_assert(1==oomfill_enabled());

    /* Cleanup attempt in case of no fork */
    oomfill_disable();
}
END_TEST

START_TEST(oomfill_config_enable_value) {
    struct rlimit limit;
    size_t l_result;
    size_t l_avail;

    /* Find available physical RAM, 0 in case of failure */
    l_avail = (sysconf(_SC_AVPHYS_PAGES) * sysconf(_SC_PAGESIZE));

    /* Configure a hard limit only if not already configured by non-forked
     * previous tests */
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    ck_assert(limit.rlim_cur==limit.rlim_max);
    l_avail=(limit.rlim_max<l_avail?limit.rlim_max:l_avail);
    oomfill_config(l_avail);

    /* Update rlimit values */
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    l_avail = limit.rlim_cur;

    /* Soft limit to half max */
    ck_assert(0==oomfill_enabled());
    l_result=oomfill_enable(l_avail/2);

    /* Check */
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    ck_assert(limit.rlim_cur==l_avail/2);
    ck_assert(limit.rlim_max==l_avail);
    ck_assert(l_result==limit.rlim_cur);
    ck_assert(1==oomfill_enabled());

    /* Cleanup attempt in case of no fork */
    oomfill_disable();
}
END_TEST

START_TEST(oomfill_config_enable_max) {
    struct rlimit limit;
    size_t l_result;
    size_t l_avail;

    /* Find available physical RAM, 0 in case of failure */
    l_avail = (sysconf(_SC_AVPHYS_PAGES) * sysconf(_SC_PAGESIZE));

    /* Configure a hard limit only if not already configured by non-forked
     * previous tests */
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    ck_assert(limit.rlim_cur==limit.rlim_max);
    l_avail=(limit.rlim_max<l_avail?limit.rlim_max:l_avail);
    oomfill_config(l_avail);

    /* Update rlimit values */
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    l_avail = limit.rlim_cur;

    /* Soft limit to hard limit */
    ck_assert(0==oomfill_enabled());
    l_result=oomfill_enable(l_avail);

    /* Check */
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    ck_assert(limit.rlim_cur==l_avail);
    ck_assert(limit.rlim_max==l_avail);
    ck_assert(l_result==limit.rlim_cur);
    ck_assert(1==oomfill_enabled());

    /* Cleanup attempt in case of no fork */
    oomfill_disable();
}
END_TEST

START_TEST(oomfill_config_enable_over) {
    struct rlimit limit;
    size_t l_avail;

    /* This test is supposed to trigger a SIGABRT(6) and will crash the
     * whole testsuite if not caught or not in a child process */
    forktest_only;

    /* Find available physical RAM, 0 in case of failure */
    l_avail = (sysconf(_SC_AVPHYS_PAGES) * sysconf(_SC_PAGESIZE));

    /* Configure a hard limit only if not already configured by non-forked
     * previous tests */
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    ck_assert(limit.rlim_cur==limit.rlim_max);
    l_avail=(limit.rlim_max<l_avail?limit.rlim_max:l_avail);
    oomfill_config(l_avail);

    /* Update rlimit values */
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    l_avail = limit.rlim_cur;

    ck_assert(0==oomfill_enabled());
    /* SIG6 expected */
    oomfill_enable(l_avail+1);
}
END_TEST

START_TEST(oomfill_config_enable_twice_same) {
    struct rlimit limit;
    size_t l_result;
    size_t l_avail;

    /* Find available physical RAM, 0 in case of failure */
    l_avail = (sysconf(_SC_AVPHYS_PAGES) * sysconf(_SC_PAGESIZE));

    /* Configure a hard limit only if not already configured by non-forked
     * previous tests */
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    ck_assert(limit.rlim_cur==limit.rlim_max);
    l_avail=(limit.rlim_max<l_avail?limit.rlim_max:l_avail);
    oomfill_config(l_avail);

    /* Update rlimit values */
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    l_avail = limit.rlim_cur;

    /* Set soft limit */
    ck_assert(0==oomfill_enabled());
    l_result=oomfill_enable(l_avail/2);

    /* Check */
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    ck_assert(limit.rlim_cur==l_avail/2);
    ck_assert(limit.rlim_max==l_avail);
    ck_assert(l_result==limit.rlim_cur);
    ck_assert(1==oomfill_enabled());

    /* Set again the same soft limit */
    l_result=oomfill_enable(l_avail/2);

    /* Check */
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    ck_assert(limit.rlim_cur==l_avail/2);
    ck_assert(limit.rlim_max==l_avail);
    ck_assert(l_result==limit.rlim_cur);
    ck_assert(1==oomfill_enabled());

    /* Cleanup attempt in case of no fork */
    oomfill_disable();
}
END_TEST

START_TEST(oomfill_config_enable_twice_up) {
    struct rlimit limit;
    size_t l_result;
    size_t l_avail;

    /* Find available physical RAM, 0 in case of failure */
    l_avail = (sysconf(_SC_AVPHYS_PAGES) * sysconf(_SC_PAGESIZE));

    /* Configure a hard limit only if not already configured by non-forked
     * previous tests */
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    ck_assert(limit.rlim_cur==limit.rlim_max);
    l_avail=(limit.rlim_max<l_avail?limit.rlim_max:l_avail);
    oomfill_config(l_avail);

    /* Update rlimit values */
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    l_avail = limit.rlim_cur;

    /* Set soft limit */
    ck_assert(0==oomfill_enabled());
    l_result=oomfill_enable(l_avail/3);

    /* Check */
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    ck_assert(limit.rlim_cur==l_avail/3);
    ck_assert(limit.rlim_max==l_avail);
    ck_assert(l_result==limit.rlim_cur);
    ck_assert(1==oomfill_enabled());

    /* Increase soft limit */
    l_result=oomfill_enable(l_avail/2);

    /* Check */
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    ck_assert(limit.rlim_cur==l_avail/2);
    ck_assert(limit.rlim_max==l_avail);
    ck_assert(l_result==limit.rlim_cur);
    ck_assert(1==oomfill_enabled());

    /* Cleanup attempt in case of no fork */
    oomfill_disable();
}
END_TEST

START_TEST(oomfill_config_enable_twice_down) {
    struct rlimit limit;
    size_t l_result;
    size_t l_avail;

    /* Find available physical RAM, 0 in case of failure */
    l_avail = (sysconf(_SC_AVPHYS_PAGES) * sysconf(_SC_PAGESIZE));

    /* Configure a hard limit only if not already configured by non-forked
     * previous tests */
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    ck_assert(limit.rlim_cur==limit.rlim_max);
    l_avail=(limit.rlim_max<l_avail?limit.rlim_max:l_avail);
    oomfill_config(l_avail);

    /* Update rlimit values */
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    l_avail = limit.rlim_cur;

    /* Set soft limit */
    ck_assert(0==oomfill_enabled());
    l_result=oomfill_enable(l_avail);

    /* Check */
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    ck_assert(limit.rlim_cur==l_avail);
    ck_assert(limit.rlim_max==l_avail);
    ck_assert(l_result==limit.rlim_cur);
    ck_assert(1==oomfill_enabled());

    /* Decrease soft limit */
    l_result=oomfill_enable(l_avail/2);

    /* Check */
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    ck_assert(limit.rlim_cur==l_avail/2);
    ck_assert(limit.rlim_max==l_avail);
    ck_assert(l_result==limit.rlim_cur);
    ck_assert(1==oomfill_enabled());

    /* Cleanup attempt in case of no fork */
    oomfill_disable();
}
END_TEST

START_TEST(oomfill_noconfig_disable) {
    /* This test is supposed to trigger a SIGABRT(6) and will crash the
     * whole testsuite if not caught or not in a child process */
    /* Furthermore, if not forked, the process might already be configured */
    forktest_only;

    ck_assert(0==oomfill_enabled());
    /* SIG6 expected */
    oomfill_disable();
}
END_TEST

START_TEST(oomfill_config_noenable_disable) {
    struct rlimit limit;
    size_t l_avail;

    /* This test is supposed to trigger a SIGABRT(6) and will crash the
     * whole testsuite if not caught or not in a child process */
    /* Furthermore, a previous test already configured the limit for the
     * process if not forked */
    forktest_only;

    /* Find available physical RAM, 0 in case of failure */
    l_avail = (sysconf(_SC_AVPHYS_PAGES) * sysconf(_SC_PAGESIZE));

    /* Configure a hard limit only if not already configured by non-forked
     * previous tests */
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    ck_assert(limit.rlim_cur==limit.rlim_max);
    l_avail=(limit.rlim_max<l_avail?limit.rlim_max:l_avail);
    oomfill_config(l_avail);

    ck_assert(0==oomfill_enabled());
    /* SIGABRT6 expected */
    oomfill_disable();
}
END_TEST

START_TEST(oomfill_config_enable_disable) {
    struct rlimit limit;
    size_t l_result;
    size_t l_avail;

    /* Find available physical RAM, 0 in case of failure */
    l_avail = (sysconf(_SC_AVPHYS_PAGES) * sysconf(_SC_PAGESIZE));

    /* Configure a hard limit only if not already configured by non-forked
     * previous tests */
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    ck_assert(limit.rlim_cur==limit.rlim_max);
    l_avail=(limit.rlim_max<l_avail?limit.rlim_max:l_avail);
    oomfill_config(l_avail);

    /* Update rlimit values */
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    l_avail = limit.rlim_cur;

    /* Set soft limit */
    ck_assert(0==oomfill_enabled());
    l_result=oomfill_enable(l_avail/2);

    /* Check */
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    ck_assert(limit.rlim_cur==l_avail/2);
    ck_assert(limit.rlim_max==l_avail);
    ck_assert(l_result==limit.rlim_cur);
    ck_assert(1==oomfill_enabled());

    /* Test disable */
    l_result=oomfill_disable();

    /* Check */
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    ck_assert(limit.rlim_cur==l_avail);
    ck_assert(limit.rlim_max==l_avail);
    ck_assert(l_result==limit.rlim_cur);
    ck_assert(0==oomfill_enabled());
}
END_TEST

START_TEST(oomfill_noconfig_noenable_fill) {
    ck_assert(0==oomfill_enabled());
    /* Should be skipped while not enabled or not configured(would be stupid) */
    oomfill_fill(0,0);
}
END_TEST

START_TEST(oomfill_config_enable_fill_minheap) {
    struct rlimit limit;
    size_t l_result;
    size_t l_avail;
    char*  l_ptr;
    size_t i=0;

    /* Find available physical RAM, 0 in case of failure */
    l_avail = (sysconf(_SC_AVPHYS_PAGES) * sysconf(_SC_PAGESIZE));

    /* Configure a hard limit only if not already configured by non-forked
     * previous tests */
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    ck_assert(limit.rlim_cur==limit.rlim_max);
    l_avail=(limit.rlim_max<l_avail?limit.rlim_max:l_avail);
    oomfill_config(l_avail);

    /* Update rlimit values */
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    l_avail = limit.rlim_cur;

    /* Set soft limit */
    l_result=oomfill_enable(l_avail/2);

    /* Check */
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    ck_assert(limit.rlim_cur==l_avail/2);
    ck_assert(limit.rlim_max==l_avail);
    ck_assert(l_result==limit.rlim_cur);

    /* Test fill */
    l_result=oomfill_fill(_i*10,2);

    /* Check remaining RAM */
    i = (long)_i*10+50;
    while ((NULL==(l_ptr=malloc(i--)))&&(i>0))
        free(l_ptr);

    if (NULL!=l_ptr)
        free(l_ptr);

    /* Cleanup mess for next tests to succeed if not forked */
    oomfill_free();
    oomfill_disable();

    /* Check */
    ck_assert(l_result<=limit.rlim_cur);
    ck_assert((long)i>=(_i*10-4));
    ck_assert((long)i<=(_i*10+100));
}
END_TEST

START_TEST(oomfill_config_enable_fill_twice) {
    struct rlimit limit;
    size_t l_avail;

    /* This test is supposed to trigger a SIGABRT(6) and will crash the
     * whole testsuite if not caught or not in a child process */
    forktest_only;

    /* Find available physical RAM, 0 in case of failure */
    l_avail = (sysconf(_SC_AVPHYS_PAGES) * sysconf(_SC_PAGESIZE));

    /* Configure a hard limit only if not already configured by non-forked
     * previous tests */
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    ck_assert(limit.rlim_cur==limit.rlim_max);
    l_avail=(limit.rlim_max<l_avail?limit.rlim_max:l_avail);
    oomfill_config(l_avail);

    /* Update rlimit values */
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    l_avail = limit.rlim_cur;

    /* Set soft limit */
    oomfill_enable(l_avail/2);

    /* Test fill */
    oomfill_fill(_i*10,2);

    /* SIGABRT6 expected */
    oomfill_fill(_i*10,2);
}
END_TEST

START_TEST(oomfill_noconfig_noenable_free) {
    /* Should simply pass */
    oomfill_free();
}
END_TEST

START_TEST(oomfill_config_noenable_free) {
    struct rlimit limit;
    size_t l_avail;

    /* Find available physical RAM, 0 in case of failure */
    l_avail = (sysconf(_SC_AVPHYS_PAGES) * sysconf(_SC_PAGESIZE));

    /* Configure a hard limit only if not already configured by non-forked
     * previous tests */
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    if (limit.rlim_max>l_avail)
        oomfill_config(l_avail);

    /* Should pass */
    oomfill_free();
}
END_TEST

START_TEST(oomfill_config_enable_free) {
    struct rlimit limit;
    size_t l_avail;

    /* This test is supposed to trigger a SIGABRT(6) and will crash the
     * whole testsuite if not caught or not in a child process */
    forktest_only;

    /* Find available physical RAM, 0 in case of failure */
    l_avail = (sysconf(_SC_AVPHYS_PAGES) * sysconf(_SC_PAGESIZE));

    /* Configure a hard limit only if not already configured by non-forked
     * previous tests */
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    ck_assert(limit.rlim_cur==limit.rlim_max);
    l_avail=(limit.rlim_max<l_avail?limit.rlim_max:l_avail);
    oomfill_config(l_avail);

    /* Update rlimit values */
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    l_avail = limit.rlim_cur;

    /* Set soft limit */
    oomfill_enable(l_avail/2);

    /* SIGABRT6 expected */
    oomfill_free();
}
END_TEST

START_TEST(oomfill_config_enable_fill_doublefree) {
    struct rlimit limit;
    size_t l_avail;

    /* This test is supposed to trigger a SIGABRT(6) and will crash the
     * whole testsuite if not caught or not in a child process */
    forktest_only;

    /* Find available physical RAM, 0 in case of failure */
    l_avail = (sysconf(_SC_AVPHYS_PAGES) * sysconf(_SC_PAGESIZE));

    /* Configure a hard limit only if not already configured by non-forked
     * previous tests */
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    ck_assert(limit.rlim_cur==limit.rlim_max);
    l_avail=(limit.rlim_max<l_avail?limit.rlim_max:l_avail);
    oomfill_config(l_avail);

    /* Update rlimit values */
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    l_avail = limit.rlim_cur;

    /* Set soft limit */
    oomfill_enable(l_avail/2);

    /* Fill RAM */
    oomfill_fill(_i*10,2);

    /* First cleanup */
    oomfill_free();

    /* SIGABRT6 expected */
    oomfill_free();
}
END_TEST

START_TEST(oomfill_noconfig_noenable_disable) {
    struct rlimit limit;
    size_t l_avail;

    /* This test is supposed to trigger a SIGABRT(6) and will crash the
     * whole testsuite if not caught or not in a child process */
    forktest_only;

    /* Find available physical RAM, 0 in case of failure */
    l_avail = (sysconf(_SC_AVPHYS_PAGES) * sysconf(_SC_PAGESIZE));

    /* Configure a hard limit only if not already configured by non-forked
     * previous tests */
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    ck_assert(limit.rlim_cur==limit.rlim_max);
    l_avail=(limit.rlim_max<l_avail?limit.rlim_max:l_avail);
    oomfill_config(l_avail);

    /* Try to disable non enabled process, Should pass */
    /* SIGABRT6 expected */
    oomfill_disable();
}
END_TEST

START_TEST(oomfill_config_enable_disable_twice) {
    struct rlimit limit;
    size_t l_avail;

    /* This test is supposed to trigger a SIGABRT(6) and will crash the
     * whole testsuite if not caught or not in a child process */
    forktest_only;

    /* Find available physical RAM, 0 in case of failure */
    l_avail = (sysconf(_SC_AVPHYS_PAGES) * sysconf(_SC_PAGESIZE));

    /* Configure a hard limit only if not already configured by non-forked
     * previous tests */
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    ck_assert(limit.rlim_cur==limit.rlim_max);
    l_avail=(limit.rlim_max<l_avail?limit.rlim_max:l_avail);
    oomfill_config(l_avail);

    /* Update rlimit values */
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    l_avail = limit.rlim_cur;

    /* Set soft limit */
    oomfill_enable(l_avail/2);

    /* First disable, already tested */
    oomfill_disable();

    /* Second disable */
    /* SIGABRT6 expected */
    oomfill_disable();
}
END_TEST

START_TEST(oomfill_config_enable_fill_disable) {
    struct rlimit limit;
    size_t l_avail;

    /* This test is supposed to trigger a SIGABRT(6) and will crash the
     * whole testsuite if not caught or not in a child process */
    forktest_only;

    /* Find available physical RAM, 0 in case of failure */
    l_avail = (sysconf(_SC_AVPHYS_PAGES) * sysconf(_SC_PAGESIZE));

    /* Configure a hard limit only if not already configured by non-forked
     * previous tests */
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    ck_assert(limit.rlim_cur==limit.rlim_max);
    l_avail=(limit.rlim_max<l_avail?limit.rlim_max:l_avail);
    oomfill_config(l_avail);

    /* Update rlimit values */
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    l_avail = limit.rlim_cur;

    /* Set soft limit */
    oomfill_enable(l_avail/2);

#ifdef ENABLE_GCOV
    /* GCOV instrumentation will SIGSEGV otherwise under this RAM pressure */
    __gcov_dump();
#endif

    /* Fill RAM */
    oomfill_fill(_i*10,2);

    /* SIGABRT6 expected */
    oomfill_disable();
}
END_TEST

START_TEST(oomfill_config_enable_fill_config) {
    struct rlimit limit;
    size_t l_avail;

    /* This test is supposed to trigger a SIGABRT(6) and will crash the
     * whole testsuite if not caught or not in a child process */
    forktest_only;

    /* Find available physical RAM, 0 in case of failure */
    l_avail = (sysconf(_SC_AVPHYS_PAGES) * sysconf(_SC_PAGESIZE));

    /* Configure a hard limit only if not already configured by non-forked
     * previous tests */
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    ck_assert(limit.rlim_cur==limit.rlim_max);
    l_avail=(limit.rlim_max<l_avail?limit.rlim_max:l_avail);
    oomfill_config(l_avail);

    /* Update rlimit values */
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    l_avail = limit.rlim_cur;

    /* Set soft limit */
    oomfill_enable(l_avail/2);

#ifdef ENABLE_GCOV
    /* GCOV instrumentation will SIGSEGV otherwise under this RAM pressure */
    __gcov_dump();
#endif

    /* Fill RAM */
    oomfill_fill(_i*10,2);

    /* SIGABRT6 expected */
    oomfill_config(0);
}
END_TEST

START_TEST(oomfill_config_enable_fill_hugeminheap) {
    struct rlimit limit;
    size_t l_avail;

    /* This test is supposed to trigger a SIGABRT(6) and will crash the
     * whole testsuite if not caught or not in a child process */
    forktest_only;

    /* Find available physical RAM, 0 in case of failure */
    l_avail = (sysconf(_SC_AVPHYS_PAGES) * sysconf(_SC_PAGESIZE));

    /* Configure a hard limit only if not already configured by non-forked
     * previous tests */
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    ck_assert(limit.rlim_cur==limit.rlim_max);
    l_avail=(limit.rlim_max<l_avail?limit.rlim_max:l_avail);
    oomfill_config(l_avail);

    /* Update rlimit values */
    ck_assert(0==getrlimit(RLIMIT_AS, &limit));
    l_avail = limit.rlim_cur;

    /* Set soft limit */
    oomfill_enable(l_avail/2);

    /* Test fill */
    /* SIGABRT6 expected */
    oomfill_fill(l_avail/2+1,2);
}
END_TEST

/************************************************************************/
/* Begining of test (potentially in child) */
void oomfill_checked_uninitialized_setup() {
    signals_catch();
    forktest_gprofdir();
}

/* End of test (potentially in child) */
void oomfill_checked_uninitialized_teardown() {
    signals_release();
}
/************************************************************************/
/* Begining of test case */
void oomfill_unchecked_setup() {
    forktest_init();
}
/* End of testcase */
void oomfill_unchecked_teardown() {
}
/************************************************************************/
Suite* oomfill_suite() {
    Suite *s;
    TCase *tc;

    s = suite_create("OOMFill");

    tc = tcase_create("Uninitialized");
    tcase_set_tags(tc,"oom");
    /* tcase_set_timeout(tc,5); */ /* seconds */
    tcase_add_checked_fixture(tc, oomfill_checked_uninitialized_setup, oomfill_checked_uninitialized_teardown);
    tcase_add_unchecked_fixture(tc, oomfill_unchecked_setup, oomfill_unchecked_teardown);
    suite_add_tcase(s, tc);

    tcase_add_test(tc, oomfill_config_once_0);
    tcase_add_test(tc, oomfill_config_once_max);
    tcase_add_test_raise_signal(tc, oomfill_config_once_over,6);
    tcase_add_test(tc, oomfill_config_once_value);
    tcase_add_test(tc, oomfill_config_twice_same);
    tcase_add_test_raise_signal(tc, oomfill_config_twice_up,6);
    tcase_add_test(tc, oomfill_config_twice_down);
    tcase_add_test_raise_signal(tc, oomfill_noconfig_enable_0,6);
    tcase_add_test_raise_signal(tc, oomfill_noconfig_enable_value,6);
    tcase_add_test_raise_signal(tc, oomfill_noconfig_enable_max,6);
    tcase_add_test_raise_signal(tc, oomfill_noconfig_enable_over,6);
    tcase_add_test(tc, oomfill_config_enable_0);
    tcase_add_test(tc, oomfill_config_enable_value);
    tcase_add_test(tc, oomfill_config_enable_max);
    tcase_add_test_raise_signal(tc, oomfill_config_enable_over,6);
    tcase_add_test(tc, oomfill_config_enable_twice_same);
    tcase_add_test(tc, oomfill_config_enable_twice_up);
    tcase_add_test(tc, oomfill_config_enable_twice_down);
    tcase_add_test_raise_signal(tc, oomfill_noconfig_disable,6);
    tcase_add_test_raise_signal(tc, oomfill_config_noenable_disable,6);
    tcase_add_test(tc, oomfill_config_enable_disable);
    tcase_add_test(tc, oomfill_noconfig_noenable_fill);
    tcase_add_loop_test(tc, oomfill_config_enable_fill_minheap,0,100);
    tcase_add_loop_test_raise_signal(tc, oomfill_config_enable_fill_twice,6,0,15);
    tcase_add_test(tc, oomfill_noconfig_noenable_free);
    tcase_add_test(tc, oomfill_config_noenable_free);
    tcase_add_test_raise_signal(tc, oomfill_config_enable_free,6);
    tcase_add_test_raise_signal(tc, oomfill_config_enable_fill_doublefree,6);
    tcase_add_test_raise_signal(tc, oomfill_noconfig_noenable_disable,6);
    tcase_add_test_raise_signal(tc, oomfill_config_enable_disable_twice,6);
    tcase_add_loop_test_raise_signal(tc, oomfill_config_enable_fill_disable,6,0,15);
    tcase_add_loop_test_raise_signal(tc, oomfill_config_enable_fill_config,6,0,15);
    tcase_add_test_raise_signal(tc, oomfill_config_enable_fill_hugeminheap,6);
    return s;
}
/* vim: set tw=80: */
